
// YO4HFU, SDA2121-2X PLL, demo sketch, frequency control via serial monitor
// Tested on Blaupunkt OPEL SC202(C), autoradio
// Version 2, 2025


#include <Wire.h>

#define IC_ADDRESS 0x64  //I2C address, A0 pin connected to GND

void setup() {
  Wire.begin();
  Serial.begin(9600);
  Serial.println("Enter the FM frequency in MHz (e.g., 100 for 100 MHz):");
}

void loop() {
  if (Serial.available() > 0) {
    String input = Serial.readStringUntil('\n');  // Read the input until a newline is encountered
    input.trim();  // Remove any leading or trailing whitespace

    float fmFrequency = input.toFloat();  // Convert the string input to a float

    if (fmFrequency > 0) {
      setToFM(fmFrequency);
    } else {
      Serial.println("Invalid frequency input. Please enter a valid number.");
    }
  }
}

void setToFM(float fmFrequency) {
  // Calculate N using the formula: N = (FMhz / 2) / 0.025. FM input has internal prescaler by 2
  int N = (fmFrequency / 2) / 0.025;
  
  // Convert N to two bytes (MSB and LSB)
  byte N_MSB = (N >> 8) & 0xFF;  // MSB
  byte N_LSB = N & 0xFF;         // LSB

  // Set Status and R Divider (sub-address 0x04)
  Wire.beginTransmission(IC_ADDRESS);
  
  // Sub-address byte: 0x04 (for status and R)
  Wire.write(0x04);  // sub-address for status and R settings

  // Status byte: 0x00 (binary 00000000), PD negative polarity, FM mode
  Wire.write(0x00);

  // R Divider: 0x00 0xA0 (160 in two bytes), 4MHz / 25KHz = 160
  Wire.write(0x00);  // R MSB
  Wire.write(0xA0);  // R LSB

  byte result = Wire.endTransmission();
  
  if (result == 0) {
    Serial.println("Status and R divider set via I2C.");
  } else {
    Serial.print("I2C Error: ");
    Serial.println(result);
  }

  // Now set N Divider (sub-address 0x0C)
  Wire.beginTransmission(IC_ADDRESS);
  
  // Sub-address byte: 0x0C (for N divider)
  Wire.write(0x0C);  // sub-address for N programming
  
  // Send N Divider as two bytes
  Wire.write(N_MSB);  // N MSB
  Wire.write(N_LSB);  // N LSB

  result = Wire.endTransmission();

  if (result == 0) {
    Serial.print("N divider set to ");
    Serial.print(N);
    Serial.println(" via I2C.");
  } else {
    Serial.print("I2C Error: ");
    Serial.println(result);
  }
}
